"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBootLoader = exports.initialize = exports.unloadBlazorPilet = exports.loadBlazorPilet = exports.unloadResource = exports.loadResourceWithSymbol = exports.loadResource = exports.getCapabilities = exports.setLanguage = exports.callNotifyLocationChanged = exports.deactivate = exports.reactivate = exports.activate = exports.destroyElement = exports.updateElement = exports.createElement = exports.setLogLevel = exports.processEvent = void 0;
const events_1 = require("./events");
const wasmLib = 'Microsoft.AspNetCore.Components.WebAssembly';
const coreLib = 'Piral.Blazor.Core';
function isDotNet6OrBelow() {
    return typeof window.Blazor?._internal?.NavigationLock === 'undefined';
}
function createBase() {
    // Nothing found, we need to guess
    const el = document.createElement('base');
    let baseUrl = el.href;
    // The main app is served by a script - but we don't know which one
    // hence we just iterate over all the local ones and use the script
    // that is served from the "shortest" route - should work almost
    // always and if not - one can always explicitely set a <base> node
    for (let i = document.scripts.length; i--;) {
        const s = document.scripts[i];
        const src = s.getAttribute('src');
        if (src && src.startsWith('/')) {
            const segEnd = src.lastIndexOf('/');
            const newUrl = src.substring(0, segEnd + 1);
            if (baseUrl.split('/').length > newUrl.split('/').length) {
                baseUrl = newUrl;
            }
        }
    }
    el.href = baseUrl;
    return document.head.appendChild(el);
}
function prepareForStartup() {
    const originalApplyHotReload = window.Blazor._internal.applyHotReload;
    const queue = [];
    const applyChanges = (api) => {
        const pilet = api.meta;
        if (pilet.config && pilet.config.blazorHotReload) {
            for (const item of queue.splice(0, queue.length)) {
                item();
            }
            window.Blazor._internal.applyHotReload = originalApplyHotReload;
        }
    };
    window.Blazor._internal.applyHotReload = function (...args) {
        queue.push(() => originalApplyHotReload.apply(this, args));
    };
    return getCapabilities().then((capabilities) => ({
        capabilities,
        applyChanges,
    }));
}
function createBlazorStarter(publicPath) {
    const root = document.body.appendChild(document.createElement('div'));
    root.style.display = 'contents';
    root.id = 'blazor-root';
    if (publicPath) {
        const baseElement = document.head.querySelector('base') || createBase();
        const originalBase = baseElement.href;
        baseElement.href = publicPath;
        return (opts) => {
            const navManager = window.Blazor._internal.navigationManager;
            //Overwrite to get NavigationManager in Blazor working, see https://github.com/smapiot/Piral.Blazor/issues/89
            navManager.navigateTo = (route, opts) => {
                if (opts.forceLoad) {
                    location.href = route;
                    return;
                }
                if (route.startsWith(location.origin) && '') {
                    // normalize "local" absolute URLs
                    route = route.substring(location.origin.length);
                }
                else if (/^https?:\/\//.test(route)) {
                    // prevent absolute URLs to be a standard navigation
                    location.href = route;
                    return;
                }
                window.Blazor.emitNavigateEvent(undefined, route, opts.replaceHistoryEntry, opts.historyEntryState);
            };
            navManager.getBaseURI = () => originalBase;
            return window.Blazor.start(opts)
                .then(prepareForStartup)
                .then(({ capabilities, applyChanges }) => {
                baseElement.href = originalBase;
                return [root, capabilities, applyChanges];
            });
        };
    }
    return (opts) => window.Blazor.start(opts)
        .then(prepareForStartup)
        .then(({ capabilities, applyChanges }) => [root, capabilities, applyChanges]);
}
function computePath() {
    try {
        throw new Error();
    }
    catch (t) {
        const e = ('' + t.stack).match(/(https?|file|ftp|chrome-extension|moz-extension):\/\/[^)\n]+/g);
        if (e) {
            return e[0].replace(/^((?:https?|file|ftp|chrome-extension|moz-extension):\/\/.+)\/[^\/]+$/, '$1') + '/';
        }
    }
    return '/';
}
function addScript(url) {
    return new Promise((resolve, reject) => {
        const script = document.createElement('script');
        script.src = url;
        script.onerror = () => reject();
        script.onload = () => resolve();
        document.body.appendChild(script);
    });
}
function sanitize(props) {
    // children is a complex thing and will (in general) not be serializable
    // hence we need to make the JSObjectReference
    if ('children' in props && typeof props.children === 'object') {
        const { children, ...rest } = props;
        return {
            children: window.DotNet.createJSObjectReference(children),
            ...rest,
        };
    }
    return props;
}
function processEvent(type, args) {
    return window.DotNet.invokeMethodAsync(coreLib, 'ProcessEvent', type, args);
}
exports.processEvent = processEvent;
function setLogLevel(logLevel) {
    return window.DotNet.invokeMethodAsync(coreLib, 'SetLogLevel', logLevel);
}
exports.setLogLevel = setLogLevel;
function createElement(moduleName, props) {
    return window.DotNet.invokeMethodAsync(coreLib, 'CreateElement', moduleName, sanitize(props));
}
exports.createElement = createElement;
function updateElement(referenceId, props) {
    return window.DotNet.invokeMethodAsync(coreLib, 'UpdateElement', referenceId, sanitize(props));
}
exports.updateElement = updateElement;
function destroyElement(referenceId) {
    return window.DotNet.invokeMethodAsync(coreLib, 'DestroyElement', referenceId);
}
exports.destroyElement = destroyElement;
function activate(moduleName, props) {
    return window.DotNet.invokeMethodAsync(coreLib, 'Activate', moduleName, sanitize(props));
}
exports.activate = activate;
function reactivate(moduleName, referenceId, props) {
    return window.DotNet.invokeMethodAsync(coreLib, 'Reactivate', moduleName, referenceId, sanitize(props)).catch(() => {
        // Apparently an older version of Piral.Blazor, which does not support this
        // discard this error silently (in the future we may print warnings here)
    });
}
exports.reactivate = reactivate;
function deactivate(moduleName, referenceId) {
    return window.DotNet.invokeMethodAsync(coreLib, 'Deactivate', moduleName, referenceId);
}
exports.deactivate = deactivate;
function callNotifyLocationChanged(url, replace, state) {
    if (isDotNet6OrBelow()) {
        return window.DotNet.invokeMethodAsync(wasmLib, 'NotifyLocationChanged', url, replace);
    }
    else {
        if (state !== undefined && typeof state !== 'string') {
            state = JSON.stringify(state);
        }
        return window.DotNet.invokeMethodAsync(wasmLib, 'NotifyLocationChanged', url, state, replace);
    }
}
exports.callNotifyLocationChanged = callNotifyLocationChanged;
function setLanguage(language) {
    return window.DotNet.invokeMethodAsync(coreLib, 'SetLanguage', language);
}
exports.setLanguage = setLanguage;
function getCapabilities() {
    return window.DotNet.invokeMethodAsync(coreLib, 'GetCapabilities').catch(() => {
        // Apparently an older version of Piral.Blazor, which does not support this
        // discard this error silently (in the future we may print warnings here)
        return [];
    });
}
exports.getCapabilities = getCapabilities;
function loadResource(url) {
    return window.DotNet.invokeMethodAsync(coreLib, 'LoadComponentsFromLibrary', url);
}
exports.loadResource = loadResource;
function loadResourceWithSymbol(dllUrl, pdbUrl) {
    return window.DotNet.invokeMethodAsync(coreLib, 'LoadComponentsWithSymbolsFromLibrary', dllUrl, pdbUrl);
}
exports.loadResourceWithSymbol = loadResourceWithSymbol;
function unloadResource(url) {
    return window.DotNet.invokeMethodAsync(coreLib, 'UnloadComponentsFromLibrary', url);
}
exports.unloadResource = unloadResource;
function loadBlazorPilet(id, data) {
    return window.DotNet.invokeMethodAsync(coreLib, 'LoadPilet', id, data);
}
exports.loadBlazorPilet = loadBlazorPilet;
function unloadBlazorPilet(id) {
    return window.DotNet.invokeMethodAsync(coreLib, 'UnloadPilet', id);
}
exports.unloadBlazorPilet = unloadBlazorPilet;
function initialize(scriptUrl, publicPath, opts = {}) {
    if (typeof opts.loadBootResource !== 'function') {
        opts.loadBootResource = (type, name, url) => type === 'dotnetjs' ? url : fetch(url, { method: 'GET', cache: 'no-cache' });
    }
    return new Promise((resolve, reject) => {
        const startBlazor = createBlazorStarter(publicPath);
        const script = document.createElement('script');
        script.src = scriptUrl;
        script.setAttribute('autostart', 'false');
        script.onerror = () => reject();
        script.onload = () => {
            Object.assign(window.Blazor, {
                emitRenderEvent: events_1.emitRenderEvent,
                emitNavigateEvent: events_1.emitNavigateEvent,
                emitPiralEvent: events_1.emitPiralEvent,
            });
            startBlazor(opts).then(resolve);
        };
        document.body.appendChild(script);
    });
}
exports.initialize = initialize;
function createBootLoader(scriptUrl, extraScriptUrls) {
    const publicPath = computePath();
    return (opts) => {
        if (typeof window.$blazorLoader === 'undefined') {
            window.dispatchEvent(new CustomEvent('loading-blazor-core'));
            // we load all satellite scripts before we initialize blazor
            window.$blazorLoader = Promise.all(extraScriptUrls.map(addScript)).then(() => initialize(scriptUrl, publicPath, opts));
        }
        return window.$blazorLoader;
    };
}
exports.createBootLoader = createBootLoader;
//# sourceMappingURL=interop.js.map